#pragma once

#include "GameTypes.h"

struct ParamInfo;
class TESObjectREFR;
class Script;
struct ScriptEventList;
class TESForm;

void Console_Print(const char * fmt, ...);

typedef bool (* _ExtractArgs)(ParamInfo * paramInfo, void * scriptData, UInt32 * arg2, TESObjectREFR * arg3, TESObjectREFR * arg4, Script * script, ScriptEventList * eventList, ...);
extern const _ExtractArgs ExtractArgs;

typedef void * (* _FormHeap_Allocate)(UInt32 size);
extern const _FormHeap_Allocate FormHeap_Allocate;

typedef void (* _FormHeap_Free)(void * ptr);
extern const _FormHeap_Free FormHeap_Free;

typedef TESForm * (* _LookupFormByID)(UInt32 id);
extern const _LookupFormByID LookupFormByID;

typedef TESForm * (* _CreateFormInstance)(UInt8 type);
extern const _CreateFormInstance CreateFormInstance;

bool IsConsoleMode();
const char * GetFullName(TESForm * baseForm);
const char* GetActorValueString(UInt32 actorValue); // not working yet

const UInt32 kMaxMessageLength = 0x4000;

bool ExtractArgsEx(ParamInfo * paramInfo, void * scriptData, UInt32 * scriptDataOffset, Script * scriptObj, ScriptEventList * eventList, ...);

// only records individual objects if there's a block that matches it
// ### how can it tell?
struct ScriptEventList
{
	struct Event
	{
		TESForm	* object;
		UInt32	eventMask;
	};

	struct VarEntry;

	struct Var
	{
		UInt32		id;
		VarEntry	* nextEntry;
		double		data;
	};

	struct VarEntry
	{
		Var			* var;
		VarEntry	* next;
	};

	Script			* m_script;
	UInt32			m_unk1;
	tList<Event>	* m_eventList;
	VarEntry		* m_vars;

	void	Dump(void);
	Var *	GetVariable(UInt32 id);
	UInt32	ResetAllVariables();
};

enum EActorVals {
	eActorVal_Aggression			= 0,
	eActorVal_Confidence			= 1,
	eActorVal_Energy				= 2,
	eActorVal_Responsibility		= 3,
	eActorVal_Mood				= 4,

	eActorVal_Strength			= 5,
	eActorVal_Perception			= 6,
	eActorVal_Endurance			= 7,
	eActorVal_Charisma			= 8,
	eActorVal_Intelligence		= 9,
	eActorVal_Agility			= 10,
	eActorVal_Luck				= 11,
	eActorVal_SpecialStart = eActorVal_Strength,
	eActorVal_SpecialEnd = eActorVal_Luck,

	eActorVal_ActionPoints		= 12,
	eActorVal_CarryWeight		= 13,
	eActorVal_CritChance			= 14,
	eActorVal_HealRate			= 15,
	eActorVal_Health				= 16,
	eActorVal_MeleeDamage		= 17,
	eActorVal_DamageResistance	= 18,
	eActorVal_PoisonResistance	= 19,
	eActorVal_RadResistance		= 20,
	eActorVal_SpeedMultiplier	= 21,
	eActorVal_Fatigue			= 22,
	eActorVal_Karma				= 23,
	eActorVal_XP					= 24,

	eActorVal_Head				= 25,
	eActorVal_Torso				= 26,
	eActorVal_LeftArm			= 27,
	eActorVal_RightArm			= 28,
	eActorVal_LeftLeg			= 29,
	eActorVal_RightLeg			= 30,
	eActorVal_Brain				= 31,
	eActorVal_BodyPartStart = eActorVal_Head,
	eActorVal_BodyPartEnd = eActorVal_Brain,

	eActorVal_Barter				= 32,
	eActorVal_BigGuns			= 33,
	eActorVal_EnergyWeapons		= 34,
	eActorVal_Explosives			= 35,
	eActorVal_Lockpick			= 36,
	eActorVal_Medicine			= 37,
	eActorVal_MeleeWeapons		= 38,
	eActorVal_Repair				= 39,
	eActorVal_Science			= 40,
	eActorVal_SmallGuns			= 41,
	eActorVal_Sneak				= 42,
	eActorVal_Speech				= 43,
	eActorVal_Throwing			= 44,
	eActorVal_Unarmed			= 45,
	eActorVal_SkillsStart = eActorVal_Barter,
	eActorVal_SkillsEnd = eActorVal_Unarmed,

	eActorVal_InventoryWeight	= 46,
	eActorVal_Paralysis			= 47,
	eActorVal_Invisibility		= 48,
	eActorVal_Chameleon			= 49,
	eActorVal_NightEye			= 50,
	eActorVal_DetectLifeRange	= 51,
	eActorVal_FireResistance		= 52,
	eActorVal_WaterBreathing		= 53,
	eActorVal_RadLevel			= 54,
	eActorVal_BloodyMess			= 55,
	eActorVal_UnarmedDamage		= 56,
	eActorVal_Assistance			= 57,

	eActorVal_ElectricResistance =  58,

	eActorVal_EnergyResistance	= 60,
	eActorVal_EMPResistance		= 61,
	eActorVal_Var1Medical		= 62,

	eActorVal_IgnoreCrippledLimbs = 72,
	eActorVal_FalloutMax = eActorVal_IgnoreCrippledLimbs,
};

class ConsoleManager
{
public:
	MEMBER_FN_PREFIX(ConsoleManager);
	DEFINE_MEMBER_FN(Print, void, 0x0071BAB0, const char * fmt, va_list args);

	ConsoleManager();
	~ConsoleManager();

	static ConsoleManager * GetSingleton(void);
};

extern bool ExtractFormatStringArgs(UInt32 fmtStringPos, char* buffer, ParamInfo * paramInfo, void * scriptDataIn, UInt32 * scriptDataOffset, Script * scriptObj, ScriptEventList * eventList, UInt32 maxParams, ...);


// A plugin author requested the ability to use OBSE format specifiers to format strings with the args
// coming from a source other than script.
// So changed ExtractFormattedString to take an object derived from following class, containing the args
// Probably doesn't belong in GameAPI.h but utilizes a bunch of stuff defined here and can't think of a better place for it
class FormatStringArgs
{
public:
	enum argType {
		kArgType_Float,
		kArgType_Form		// TESForm*
	};

	virtual bool Arg(argType asType, void * outResult) = 0;	// retrieve next arg
	virtual bool SkipArgs(UInt32 numToSkip) = 0;			// skip specified # of args
	virtual bool HasMoreArgs() = 0;
	virtual std::string GetFormatString() = 0;						// return format string
};

// concrete class used for extracting script args
class ScriptFormatStringArgs : public FormatStringArgs
{
public:
	virtual bool Arg(argType asType, void* outResult);
	virtual bool SkipArgs(UInt32 numToSkip);
	virtual bool HasMoreArgs();
	virtual std::string GetFormatString();

	ScriptFormatStringArgs(UInt32 _numArgs, UInt8* _scriptData, Script* _scriptObj, ScriptEventList* _eventList);
	UInt32 GetNumArgs();
	UInt8* GetScriptData();

private:
	UInt32			numArgs;
	UInt8			* scriptData;
	Script			* scriptObj;
	ScriptEventList	* eventList;
	std::string fmtString;
};