#include "nvse/GameTiles.h"
#include "nvse/Utilities.h"
#include "nvse/GameAPI.h"
#include <string>

typedef NiTMapBase <const char *, int>	TraitNameMap;
TraitNameMap	* g_traitNameMap = (TraitNameMap *)0x011EAF8C;

const _TraitNameToID TraitNameToID = (_TraitNameToID)0x009FBE50;

UInt32 Tile::TraitNameToID(const char * traitName)
{
	return ::TraitNameToID(traitName);
}

Tile::Value * Tile::GetValue(UInt32 typeID)
{
	// values are sorted so could use binary search but these are not paricularly large arrays
	for(UInt32 i = 0; i < values.size; i++)
	{
		Tile::Value * val = values[i];
		if(val && val->id == typeID)
			return val;
	}

	return NULL;
}

Tile::Value * Tile::GetValue(const char * valueName)
{
	UInt32 typeID = TraitNameToID(valueName);

	return GetValue(typeID);
}

Tile * Tile::GetChild(const char * childName)
{
	Tile * child = NULL;

	for(tList<ChildNode>::Iterator iter = childList.Begin(); !iter.End(); ++iter)
	{
		if(*iter && iter->child && !_stricmp(iter->child->name.m_data, childName))
		{
			child = iter->child;
			break;
		}
	}

	return child;
}

Tile::Value * Tile::GetComponentValue(const char * componentPath)
{
	Tokenizer tokens(componentPath, "\\/");
	std::string curToken;
	Tile * parentTile = this;

	while(tokens.NextToken(curToken) != -1 && parentTile)
	{
		DEBUG_PRINT("childName: %s", curToken.c_str());

		Tile * child = parentTile->GetChild(curToken.c_str());
		if(!child)
		{
			// didn't find child; is this last token?
			if(tokens.NextToken(curToken) == -1)
				return parentTile->GetValue(curToken.c_str());
			else	// nope, error
				return NULL;
		}
		else
			parentTile = child;
	}

	// couldn't locate one of the child nodes
	return NULL;
}

std::string Tile::GetQualifiedName(void)
{
	std::string qualifiedName;

	//if(parent && !parent->GetFloatValue(kTileValue_class, &parentClass))	// i.e., parent is not a menu
	if(parent)
		qualifiedName = parent->GetQualifiedName() + "\\";

	qualifiedName += name.m_data;

	return qualifiedName;
}

void Tile::Dump(void)
{
	_MESSAGE("%s", name.m_data);
	gLog.Indent();

	_MESSAGE("values:");

	gLog.Indent();
	
	for(UInt32 i = 0; i < values.size; i++)
	{
		Value		* val = values[i];
		const char	* traitName = TraitIDToName(val->id);
		char		traitNameIDBuf[16];

		if(!traitName)
		{
			sprintf_s(traitNameIDBuf, "%08X", val->id);
			traitName = traitNameIDBuf;
		}

		if(val->str)
			_MESSAGE("%s: %s", traitName, val->str);
		else if(val->action)
			_MESSAGE("%s: action %08X", traitName, val->action);
		else
			_MESSAGE("%s: %f", traitName, val->num);
	}

	gLog.Outdent();

	for(tList <ChildNode>::Iterator iter = childList.Begin(); !iter.End(); ++iter)
	{
		ChildNode	* node = iter.Get();
		if(node)
		{
			Tile	* child = node->child;
			if(child)
			{
				child->Dump();
			}
		}
	}

	gLog.Outdent();
}

void Debug_DumpTraits(void)
{
	for(UInt32 i = 0; i < g_traitNameMap->m_numBuckets; i++)
	{
		for(TraitNameMap::Bucket * bucket = g_traitNameMap->m_buckets[i]; bucket; bucket = bucket->next)
		{
			_MESSAGE("%s %d", bucket->key, bucket->data);
		}
	}
}

// not a one-way mapping, so we just return the first
// also this is slow and sucks
const char * TraitIDToName(int id)
{
	for(UInt32 i = 0; i < g_traitNameMap->m_numBuckets; i++)
		for(TraitNameMap::Bucket * bucket = g_traitNameMap->m_buckets[i]; bucket; bucket = bucket->next)
			if(bucket->data == id)
				return bucket->key;

	return NULL;
}
