#include "nvse/GameUI.h"

UInt8	* g_MenuVisibilityArray = (UInt8 *)0x011EAD27;
NiTArray <TileMenu *> * g_TileMenuArray = (NiTArray <TileMenu *> *)0x011EB1A0;

enum
{
	// ###

	kMenuType_Min = 0x3E9,
	kMenuType_Max = 0x43C,
};

InterfaceManager * InterfaceManager::GetSingleton(void)
{
	return *(InterfaceManager **)0x011D0820;
}

bool InterfaceManager::IsMenuVisible(UInt32 menuType)
{
	if((menuType >= kMenuType_Min) && (menuType <= kMenuType_Max))
		return g_MenuVisibilityArray[menuType] != 0;

	return false;
}

Menu * InterfaceManager::GetMenuByType(UInt32 menuType)
{
	if((menuType >= kMenuType_Min) && (menuType <= kMenuType_Max))
	{
		TileMenu * tileMenu = g_TileMenuArray->Get(menuType - kMenuType_Min);
		if (tileMenu)
			return tileMenu->menu;
	}

	return NULL;
}

Tile::Value* InterfaceManager::GetMenuComponentValue(char * componentPath)
{
	// path is of format "MenuType/tile/tile/.../traitName" following hierarchy defined in menu's xml

	// get menu name - stored by game as "&MENUNAME;" so need to fix it up
	char* slashPos = strpbrk(componentPath, "\\/");
	if(slashPos)
	{
		*slashPos = 0;

		char menuName[0x20];

		UInt32 len = strlen(componentPath);
		strcpy_s(menuName + 1, len + 1, componentPath);
		menuName[len + 1] = ';';
		menuName[0] = '&';
		menuName[len + 2] = '\0';

		UInt32 menuID = Tile::TraitNameToID(menuName);
		if(menuID >= kMenuType_Min && menuID <= kMenuType_Max)
		{
			TileMenu	* tileMenu = g_TileMenuArray->Get(menuID - kMenuType_Min);

			if(tileMenu)
				return tileMenu->GetComponentValue(slashPos + 1);
		}
	}

	return NULL;
}

void Debug_DumpMenus(void)
{
	for(UInt32 i = 0; i < g_TileMenuArray->length; i++)
	{
		TileMenu	* tileMenu = g_TileMenuArray->Get(i);

		if(tileMenu)
		{
			_MESSAGE("menu %d:");
			gLog.Indent();

			tileMenu->Dump();

			gLog.Outdent();
		}
	}
}

